#!/usr/bin/python

def bernstein(y, t):
	n = len(y)-1
	t = 1.0*t
	s = 1-t
	p = y[0]
	k = 1
	c = n*t
	for i in range(0,n):
		p = p*s + c*y[k]
		c = c*(n-k)*t/(k+1)
		k = k+1
	return(p)

def delta(y):
	n = len(y)-1
	d = []
	for i in range(n):
		d.append(y[i+1]-y[i])
	return d

def sigma(y):
	s = [0]
	sum = 0
	for x in y:
		sum += x
		s.append(sum)
	return s

# returns the array of the component Bezier arrays
# args = p or p, n
def subdivide(*args):
	if len(args) == 1:
		p = args[0]
		if len(p) == 4: # a Bezier segment
			p0 = p[0]; p1 = p[1]; p2 = p[2]; p3 = p[3]
			p01 = interpolate(p0, p1, 0.5)
			p12 = interpolate(p1, p2, 0.5)
			p23 = interpolate(p2, p3, 0.5)
			p012 = interpolate(p01, p12, 0.5)
			p123 = interpolate(p12, p23, 0.5)
			p0123 = interpolate(p012, p123, 0.5)
			return [[p0, p01, p012, p0123], [p0123, p123, p23, p3]]
		else: # a line
			p0 = p[0]; p1 = p[1]
			p01 = interpolate(p0, p1, 0.5)
			return [[p0, p01], [p01, p1]]
	else:
		p = [ args[0] ]
		n = args[1]
		for i in range(n): 
			# facing an array of 2^i components
			# turn into an array of 2^i+1 components
			m = len(p)
			q = []
			for j in range(m):
				r = subdivide(p[i])
				q.append(r[0])
				q.append(r[1])
			p = q
	return(p)

def interpolate(p, q, t):
	s = 1-t
	r = []
	for i in range(len(p)):
		r.append(s*p[i]+t*q[i])
	return(r)

def choose(n, k):
	p = 1
	j = 1
	for i in range(k):
		p *= n
		p /= j
		n -= 1
		j += 1
	return(p)

def power(x, n):
	p = 1
	for i in range(n):
		p *= x
	return(p)

def coefficient(x, n, k):
	x = 1.0*x
	p = power(1-x,n)
	j = 1
	for i in range(k):
		p *= (x/1-x)
		p *= n
		p /= j
		n -= 1
		j += 1

def approximation(n, f, x):
	s = 0
	for i in range(n+1):
		s += f(i*1.0/n)*choose(n, i)*power(x,i)*power(1-x,n-i)
	return(s)
		

"""

# ===============================================

import sys
n = int(sys.argv[1])

for i in range(n+1):
	print choose(n,i)

"""
	
	



